import argparse
import datetime
import json
import sys
import wizard_domaininfo

try:
    from collections import OrderedDict
except ImportError as e:
    from ordereddict import OrderedDict

parser = argparse.ArgumentParser(description="Retrieves and parses WHOIS data for a domain name using "
                                             "wizard_domaininfo.")
parser.add_argument("-r", "--raw", action="store_true",
                    help="Outputs raw WHOIS data and doesn't attempt to parse it. Segments are separated by a "
                         "double-dash (--).")
parser.add_argument("-j", "--json", action="store_true",
                    help="Outputs structured WHOIS data in JSON format, according to the wizard_domaininfo API.")
parser.add_argument("-f", "--file", action="store",
                    help="Loads and parses raw double-dash-delimited WHOIS data from a specified file, instead of "
                         "contacting a WHOIS server.",
                    default=None)
parser.add_argument("domain", nargs=1)
args = parser.parse_args()


def json_fallback(obj):
    if isinstance(obj, datetime.datetime):
        return obj.isoformat()
    else:
        return obj


if args.file is None:
    data, server_list = wizard_domaininfo.net.get_whois_raw(args.domain[0], with_server_list=True)
else:
    server_list = []
    with open(args.file, "r") as f:
        data = f.read().split("\n--\n")

if args.raw:
    print(b"\n--\n".join([x.encode("utf-8") for x in data]))
else:
    if len(server_list) > 0:
        parsed = wizard_domaininfo.parse.parse_raw_whois(data, normalized=True, never_query_handles=False,
                                                         handle_server=server_list[-1])
    else:
        parsed = wizard_domaininfo.parse.parse_raw_whois(data, normalized=True)

    if args.json:
        print(json.dumps(parsed, default=json_fallback))
    else:
        data_map = OrderedDict({})

        # This defines the fields shown in the output
        data_map["id"] = ("Domain ID", 1)
        data_map["status"] = ("Status", 1)
        data_map["registrar"] = ("Registrar", 1)
        data_map["creation_date"] = ("Registration date", 1)
        data_map["expiration_date"] = ("Expiration date", 1)
        data_map["updated_date"] = ("Last update", 1)
        data_map["nameservers"] = ("Name server", "+")
        data_map["emails"] = ("E-mail address", "+")

        widest_label = 0
        for key, value in data_map.items():
            if len(value[0]) > widest_label:
                widest_label = len(value[0])

        for key, value in data_map.items():
            if key in parsed and parsed[key] is not None:
                label = value[0] + (" " * (widest_label - len(value[0]))) + " :"
                if value[1] == 1:
                    print("%s %s" % (label, parsed[key][0]))
                elif value[1] == "+":
                    for item in parsed[key]:
                        print("%s %s" % (label, item))

        if parsed["contacts"] is not None:
            # This defines the contacts shown in the output
            contacts_map = OrderedDict({})
            contacts_map["registrant"] = "Registrant"
            contacts_map["tech"] = "Technical Contact"
            contacts_map["admin"] = "Administrative Contact"
            contacts_map["billing"] = "Billing Contact"

            # This defines the contact data shown in the output
            data_map = OrderedDict({})
            data_map["handle"] = "NIC handle"
            data_map["name"] = "Name"
            data_map["organization"] = "Organization"
            data_map["street"] = "Street address"
            data_map["postalcode"] = "Postal code"
            data_map["city"] = "City"
            data_map["state"] = "State / Province"
            data_map["country"] = "Country"
            data_map["email"] = "E-mail address"
            data_map["phone"] = "Phone number"
            data_map["fax"] = "Fax number"
            data_map["creationdate"] = "Created"
            data_map["changedate"] = "Last changed"

            for contact in contacts_map:
                if parsed["contacts"][contact] is not None:
                    contact_data = parsed["contacts"][contact]

                    print("\n" + contacts_map[contact])

                    for key, value in data_map.items():
                        if len(value) > widest_label:
                            widest_label = len(value)

                    for key, value in data_map.items():
                        if key in contact_data and contact_data[key] is not None:
                            label = "    " + value + (" " * (widest_label - len(value))) + " :"
                            if sys.version_info < (3, 0):
                                if type(contact_data[key]) == str:
                                    actual_data = contact_data[key].decode("utf-8")
                                elif type(contact_data[key]) == datetime.datetime:
                                    actual_data = unicode(contact_data[key])
                                else:
                                    actual_data = contact_data[key]
                            else:
                                actual_data = str(contact_data[key])
                            if "\n" in actual_data:  # Indent multi-line values properly
                                lines = actual_data.split("\n")
                                actual_data = "\n".join(
                                    [lines[0]] + [(" " * (widest_label + 7)) + line for line in lines[1:]])
                            print("%s %s" % (label, actual_data))



import json
import timeit
import aiodns
import asyncio
from datetime import date, datetime
from collections import defaultdict
import wizard_domaininfo
from wizard_domaininfo.utils import get_dmarc_record, get_a_record, get_soa_record, get_aaaa_record, get_ns_records, \
    get_txt_records, get_hostname_from_ip, get_rdns_from_ip, get_domain_rdap_info, get_domain_whois_info_legacy, \
    parse_date_convert, get_dkim_records, check_domain_expired, get_cname_record, get_mx_records

TIMEOUT = 1.0  # timeout in seconds
wizard_domaininfo.net.socket.setdefaulttimeout(TIMEOUT)
url = ''


class DomainInfo:
    DEFAULT_TIMEOUT = 1  # seconds
    rdapbootstrapurl = 'https://www.rdap.net/'
    wizard_domaininfo.net.socket.setdefaulttimeout(DEFAULT_TIMEOUT)
    default_resolvers = ['1.1.1.1', '8.8.8.8', '1.0.0.1', '8.8.4.4']

    def __init__(self, domain):
        self.name = domain
        self.domain = domain
        # Setup dictionary and defaults
        self.domain = domain.lower()
        self.url = 'http://' + self.domain
        self.domain_dict = defaultdict(set)
        self.domain_whois = defaultdict(set)
        self.registrar = ''
        self.registration = ''
        self.expiration = ''
        self.status = ''
        self.soa = {}
        self.whois_statuses = []
        # Setup lists variables
        # Nameservers with A record lookups
        self.whois_nameservers = []
        self.domain_nameservers = []
        # Domain WWW: A, AAA, CNAME values
        self.domain_www = []
        # Domain MX Records list
        self.domain_mx = []
        # Domain TXT type records
        self.domain_txt = []
        # Nameserver lists without IP's
        self.whois_ns = []
        self.ns = []
        # Abort DNS lookups when no valid DNS NS found to prevent lockups
        self.dns_lookup_continue = ''
        # Force DNS lookups when using custom resolver: todo
        self.dns_lookup_force = ''
        # Domain Expired
        self.expired = ''
        # Domain DNS Dictionary
        self.dns = defaultdict(set)
        # Whois and DNS NS agree on Nameserver names
        self.auth_ns_match = ''
        # Sender Policy Framework
        self.spf = ''
        # DomainKeys Identified Mail (DKIM)
        self.dkim = []
        # Domain-based Message Authentication, Reporting & Conformance (DMARC)
        self.dmarc = ''
        # Holds values for detected WAF's/CDN/Proxy like Sucuri/Cloudflare/Quic.cloud
        self.waf = ''
        # DNSSEC aka SecureDNS status of domain
        self.dnssec = ''
        # Setup asyncio
        # self.loop = asyncio.get_event_loop()
        # Needed due to running this in another thread without a previous event loop
        # self.loop = asyncio.set_event_loop(asyncio.new_event_loop())
        self.resolver = aiodns.DNSResolver(loop=self.loop)
        self.custom_resolvers = []
        self.resolver.nameservers = self.default_resolvers
        self.rdap_url = self.rdapbootstrapurl + "domain/" + domain

        # Initialize Whois and DNS
        self.get_whois_domain()
        self.check_expiration()
        self.get_domain_dns()
        self.check_auth_nameservers_match()

    async def query(self, name, query_type):
        return await self.resolver.query(name, query_type)

    def get_domain_whois_info(self):

        try:
            self.domain_whois = get_domain_whois_info_legacy(self.domain)
            # print(self.domain_whois)
        except:
            return False
            # pass

        # "WHOIS": {"status": "['client delete prohibited', 'server transfer prohibited', 'server update prohibited']"
        try:
            # print(str(self.domain_whois['status'][0]).rsplit())
            whois_status = self.domain_whois['status']
            # print(whois_status)
            for status in whois_status:
                status = status.rsplit()
                # print(status[0])
                self.whois_statuses.append(status[0])
            self.status = self.whois_statuses[0]
        except:
            self.status = 'No Status Found'
            pass

        # Source "WHOIS": {"registration": "1997-09-15T04:00:00Z", "expiration": "2028-09-14T04:00:00Z"}
        try:
            self.registration = parse_date_convert(str(self.domain_whois['creation_date'][0]))
        except:
            pass

        try:
            self.expiration = parse_date_convert(str(self.domain_whois['expiration_date'][0]))
        except:
            self.expired = False
            self.expiration = 'No Expiration Found'
            pass

        # "WHOIS": {"registrar": "MarkMonitor Inc."}
        try:
            self.registrar = str(self.domain_whois['registrar'][0])
        except:
            pass

        # "WHOIS": {"secureDNS": {"delegationSigned": false}}
        try:
            domainwhois_dnssec_raw = str(self.domain_whois['raw']).split('DNSSEC: ', 1)[1]
            # print(domainwhois_dnssec_raw)
            if "signedDelegation" in domainwhois_dnssec_raw:
                self.dnssec = True
            elif "unsigned" in domainwhois_dnssec_raw:
                self.dnssec = False
        except:
            pass

        #  "WHOIS": {"nameservers": [["NS1.GOOGLE.COM", "216.239.32.10"], ["NS2.GOOGLE.COM", "216.239.34.10"]]}
        try:
            for nameserver in self.domain_whois['nameservers']:
                # print(ns)
                ns = nameserver.lower()
                ip = get_a_record(ns)
                # print(ns, ip)
                self.whois_nameservers.append([str(ns), str(ip)])
                self.whois_ns.append(ns)
        except:
            pass

    def check_expiration(self):
        if not self.expiration == 'No Expiration Found':
            if check_domain_expired(self.expiration):
                self.expired = True
            else:
                self.expired = False

    def create_domain_dict_rdap(self):
        # "WHOIS": {"registrar": "MarkMonitor Inc."}
        try:
            self.registrar = str(self.domain_whois["entities"][0]['vcardArray'][1][1][3])
        except:
            pass

        # "WHOIS": {"status": "['client delete prohibited', 'server transfer prohibited', 'server update prohibited']"
        try:
            whois_status = self.domain_whois['status']
            for status in whois_status:
                self.whois_statuses.append(status)
            self.status = self.whois_statuses[0]
        except:
            pass

        # "WHOIS": {"registration": "1997-09-15T04:00:00Z", "expiration": "2028-09-14T04:00:00Z"}
        try:
            for event in self.domain_whois['events']:
                # print(event)
                event_action = event['eventAction']
                event_date = parse_date_convert(str(event["eventDate"]))
                if event_action == 'registration':
                    self.registration = event_date
                elif event_action == 'expiration':
                    self.expiration = event_date
        except:
            pass

        # "WHOIS": {"secureDNS": {"delegationSigned": false}}
        try:
            self.dnssec = self.domain_whois['secureDNS']['delegationSigned']
        except:
            pass

        #  "WHOIS": {"nameservers": [["NS1.GOOGLE.COM", "216.239.32.10"], ["NS2.GOOGLE.COM", "216.239.34.10"]]}
        try:
            for nameserver in self.domain_whois['nameservers']:
                # print(nameserver['ldhName'])
                ns = nameserver['ldhName'].lower()
                # print(result)
                ip = get_a_record(ns)
                # print(ns, ip)
                self.whois_nameservers.append([ns, ip])
                self.whois_ns.append(ns)
        except:
            pass

    def get_domain_dns(self):
        try:
            res_ns = get_ns_records(self.domain)
            for ns in res_ns:
                ip = get_a_record(ns)
                self.ns.append(ns)
                self.domain_nameservers.append([ns, ip])
                if "cloudflare" in ns:
                    # print("Cloudflare: FullZone detected")
                    self.waf = 'Cloudflare: FullZone detected'
            self.dns_lookup_continue = True
        except:
            # No point in checking for DNS is there are no nameservers with IP's responding
            self.dns_lookup_continue = False
            pass

        if self.dns_lookup_continue or self.dns_lookup_force:
            try:
                # SOA query the host's DNS
                self.soa = get_soa_record(self.domain)
                if "cloudflare" in self.soa['nsname']:
                    self.waf = 'Cloudflare: FullZone detected'
            except:
                pass

            try:
                self.dkim = get_dkim_records(self.domain)
                for key in self.dkim:
                    self.domain_txt.append(['TXT', str(key), self.dkim[key]])
            except:
                pass

            try:
                self.dmarc = get_dmarc_record(self.domain)
                if self.dmarc is not None:
                    self.domain_txt.append(['TXT', str(f'_dmarc.{self.domain}'), str(self.dmarc)])
            except:
                pass

            try:
                # WWW query the host's DNS
                www_name = 'www.' + self.domain
                res_cname = get_cname_record(www_name)
                if res_cname is not None:
                    self.domain_www.append(['CNAME', str(www_name), str(res_cname)])
                    if "cloudflare" in res_cname:
                        self.waf = 'Cloudflare: CNAME detected'
                    if "quic.cloud" in res_cname:
                        self.waf = "QUIC.cloud CDN: CNAME detected"
            except:
                pass

            try:
                www_name = 'www.' + self.domain
                domain_www_a = get_a_record(www_name)
                if domain_www_a is not None:
                    self.domain_www.append(['A', str(www_name), str(domain_www_a)])
            except:
                pass

            try:
                domain_a = get_a_record(self.domain)
                if domain_a is not None:
                    self.domain_www.append(['A', str(self.domain), str(domain_a)])
            except:
                pass

            try:
                domain_aaaa = get_aaaa_record(self.domain)
                if domain_aaaa is not None:
                    self.domain_www.append(['AAAA', str(self.domain), str(domain_aaaa)])
            except:
                pass

            try:
                # MX query the host's DNS
                res_mx = get_mx_records(self.domain)
                for elem in res_mx:
                    self.domain_mx.append(['MX', str(elem.host), str(elem.priority)])
            except:
                pass

            try:
                res_txt = self.loop.run_until_complete(self.resolver.query(self.domain, 'TXT'))
                for elem in res_txt:
                    # print(str(elem.text))
                    self.domain_txt.append(['TXT', str(self.domain), str(elem.text)])
                    if 'v=spf' in str(elem.text):
                        self.spf = str(elem.text)
            except:
                pass

            domain_dict = {
                'domain': self.domain,
                'rdapurl': self.rdap_url,
                'WHOIS': {
                    'Registrar': self.registrar,
                    'registration': self.registration,
                    'expiration': self.expiration,
                    'secureDNS': self.dnssec,
                    'status': self.whois_statuses,
                    'nameservers': self.whois_nameservers,

                },
                'DNS': {
                    'SOA': self.soa['SOA'],
                    'NS': self.domain_nameservers,
                    'WWW': self.domain_www,
                    'MX': self.domain_mx,
                    'TXT': self.domain_txt,
                }
            }

            print(domain_dict)
            self.domain_dict = domain_dict

    def get_whois_domain(self):
        self.domain_whois = get_domain_rdap_info(self.domain)
        if self.domain_whois:
            self.create_domain_dict_rdap()
        else:
            self.rdap_url = 'NA'
            self.get_domain_whois_info()

    def check_auth_nameservers_match(self):
        if sorted(self.whois_ns) == sorted(self.ns):
            # print('Authoritative NS and DNS nameservers match')
            self.auth_ns_match = True
        else:
            self.auth_ns_match = False


# How to use
def check_domaininfo():
    domain = DomainInfo('wizardassistant.com')
    print(f"{domain.domain}'s is expired {domain.expired} ")
    print(f"{domain.domain}'s registrar is {domain.registrar} ")
    print(f"Whois Nameservers: {domain.whois_nameservers} ")
    print(f"{domain.domain}'s registrar status: {domain.status}")
    # print(type(domain.status))
    print(f"Domain's WAF/CDN Status: {domain.waf}")
    print(f"Domain's DNSSEC Status: {domain.dnssec}")
    print(f"WWW records: {domain.domain_www}")
    print(f"SOA record: {domain.soa}")
    print(f"MX records: {domain.domain_mx}")
    print(f"DNS Nameservers: {domain.ns} ")
    print(f"Domain's SPF: {domain.spf} ")
    print(f"Domain's DKIM: {domain.dkim} ")
    print(f"Domain's DMARC: {domain.dmarc} ")
    print(f"Domain Expiration: {domain.expiration} ")
    print(f"Whois Nameservers: {domain.whois_ns} ")
    print(f"DNS Nameservers: {domain.ns} ")
    print(f"Auth WHOIS and DNS Nameservers match: {domain.auth_ns_match} ")
    print(f"WAF check: {domain.waf} ")
    # for key, value in domain.dns.items():
    #    print(key, ':', value)
    print(json.dumps(domain.domain_dict, indent=4, sort_keys=False))


# elapsed_time = timeit.timeit(check_domaininfo, number=1)/1
# print("DNS Lookup took: ", elapsed_time)

# check_domaininfo()



from datetime import datetime
import aiodns
import asyncio
import json
import requests
from dateutil.parser import parse as date_parse
from requests.exceptions import Timeout, ConnectionError
from requests.packages.urllib3.util.retry import Retry
from requests.adapters import HTTPAdapter
from wizard_domaininfo import get_whois, net as wizard_whois, parse


# https://findwork.dev/blog/advanced-usage-python-requests-timeouts-retries-hooks/
class TimeoutHTTPAdapter(HTTPAdapter):
    def __init__(self, *args, **kwargs):
        """
        The HTTPAdapter class combines retries and timeouts for the requests library.

        Args:
            *args ():
            **kwargs ():
        """
        DEFAULT_TIMEOUT = 1
        self.timeout = DEFAULT_TIMEOUT
        if "timeout" in kwargs:
            self.timeout = kwargs["timeout"]
            del kwargs["timeout"]
        super().__init__(*args, **kwargs)

    def send(self, request, **kwargs):
        """Send a request via TimeoutHTTPAdapter.

        Args:
            request (): request
            **kwargs (): kwargs

        Returns: Boolean

        """
        timeout = kwargs.get("timeout")
        if timeout is None:
            kwargs["timeout"] = self.timeout
        return super().send(request, **kwargs)


http = requests.Session()
# Mount  TimeoutHTTP adapter with retries it for both http and https usage
adapter = TimeoutHTTPAdapter(timeout=2.5)
retries = Retry(total=1, backoff_factor=1, status_forcelist=[429, 500, 502, 503, 504])
http.mount("https://", TimeoutHTTPAdapter(max_retries=retries))
http.mount("http://", TimeoutHTTPAdapter(max_retries=retries))

TIMEOUT = 1.0  # timeout in seconds
rdapbootstrapurl = "https://www.rdap.net/"
url = ''

wizard_whois.socket.setdefaulttimeout(TIMEOUT)


def parse_date_convert(date, fmt=None):
    """Parse date from string and convert to desired format.

    :param date: Date to parse
    :type date: str
    :param fmt: Datetime format to convert to
    :type fmt: str
    :return: Returns parsed and converted datetime in a string otherwise `None`
    :rtype: str
    """
    if fmt is None:
        fmt = '%Y-%m-%d %H:%M:%S'
    if date is None:
        return False
    get_date_obj = date_parse(str(date))
    fmt_date = str(get_date_obj.strftime(fmt))
    return fmt_date


def check_http_reachable(domain_name):
    """Check if the domain is reachable via http.

    :param domain_name: Domain name to check is reachable.
    :type domain_name: str
    :return: Returns `True` if connectable, `False` otherwise
    :rtype: bool
    """
    global url
    try:
        if "http" not in domain_name:
            url = "http://" + domain_name
        else:
            url = domain_name
        http.get(url)
        return True
    except requests.exceptions.ConnectionError:
        print(f"URL {url} not reachable")
        return False


def check_url_ssl(domain_name):
    """Check if the domain has a valid SSL certificate.

    :param domain_name: Domain name to check SSL for.
    :type domain_name: str
    :return: Returns `True` if valid, `False` otherwise
    :rtype: bool
    """
    global url
    try:
        if "http" not in domain_name:
            url = "https://" + domain_name
        else:
            url = domain_name
        http.get(url, verify=True)
        return True
    except requests.exceptions.ConnectionError:
        print(f"URL {url} not reachable or SSL expired")
        return False


loop = asyncio.get_event_loop()
resolver = aiodns.DNSResolver(loop=loop)


async def query(name, query_type):
    return await resolver.query(name, query_type)


def get_hostname_from_ip(ip):
    """Basic get RDNS via PTR lookup.

    :param ip: IP address to lookup
    :type ip: str
    :return: Returns `hostname` if found, or empty string '' otherwise
    :rtype: str
    """
    try:
        reverse_name = ".".join(reversed(ip.split("."))) + ".in-addr.arpa"
        coro = query(reverse_name, "PTR")
        result = loop.run_until_complete(coro)
        return result.name
    except:
        return ""


def get_rdns_from_ip(ip):
    """Basic get RDNS via gethostbyaddr.

    :param ip: IP address to lookup
    :type ip: str
    :return: Returns `hostname` if found, or empty string '' otherwise
    :rtype: str
    """
    try:
        coro = resolver.gethostbyaddr(ip)
        result = loop.run_until_complete(coro)
        return result.name
    except:
        return ""


def get_domain_whois_info_legacy(domain):
    """Do WHOIS lookup via legacy method via whois socket.

    :param domain: Domain to lookup whois for
    :type domain: str
    :return: Dictionary of domain whois information
    :rtype: dict, bool
    :Example:
    >>> print(get_domain_whois_info_legacy('google.com'))
    `{'id': ['2138514_DOMAIN_COM-VRSN'], 'status': ['clientDeleteProhibited https://icann.org/epp#clientDeleteProhibited', 'clientTransferProhibited https://icann.org/epp#clientTransferProhibited', 'clientUpdateProhibited https://icann.org/epp#clientUpdateProhibited', 'serverDeleteProhibited https://icann.org/epp#serverDeleteProhibited', 'serverTransferProhibited https://icann.org/epp#serverTransferProhibited', 'serverUpdateProhibited https://icann.org/epp#serverUpdateProhibited'], 'creation_date': [datetime.datetime(1997, 9, 15, 4, 0)], 'expiration_date': [datetime.datetime(2028, 9, 14, 4, 0)], 'updated_date': [datetime.datetime(2019, 9, 9, 15, 39, 4)], 'registrar': ['MarkMonitor Inc.'], 'whois_server': ['whois.markmonitor.com'], 'nameservers': ['NS1.GOOGLE.COM', 'NS2.GOOGLE.COM', 'NS3.GOOGLE.COM', 'NS4.GOOGLE.COM'], 'emails': ['abusecomplaints@markmonitor.com'], 'contacts': {'registrant': None, 'tech': None, 'admin': None, 'billing': None}, 'raw': ['   Domain Name: GOOGLE.COM\n   Registry Domain ID: 2138514_DOMAIN_COM-VRSN\n   Registrar WHOIS Server: whois.markmonitor.com\n   Registrar URL: http://www.markmonitor.com\n   Updated Date: 2019-09-09T15:39:04Z\n   Creation Date: 1997-09-15T04:00:00Z\n   Registry Expiry Date: 2028-09-14T04:00:00Z\n   Registrar: MarkMonitor Inc.\n   Registrar IANA ID: 292\n   Registrar Abuse Contact Email: abusecomplaints@markmonitor.com\n   Registrar Abuse Contact Phone: +1.2083895740\n   Domain Status: clientDeleteProhibited https://icann.org/epp#clientDeleteProhibited\n   Domain Status: clientTransferProhibited https://icann.org/epp#clientTransferProhibited\n   Domain Status: clientUpdateProhibited https://icann.org/epp#clientUpdateProhibited\n   Domain Status: serverDeleteProhibited https://icann.org/epp#serverDeleteProhibited\n   Domain Status: serverTransferProhibited https://icann.org/epp#serverTransferProhibited\n   Domain Status: serverUpdateProhibited https://icann.org/epp#serverUpdateProhibited\n   Name Server: NS1.GOOGLE.COM\n   Name Server: NS2.GOOGLE.COM\n   Name Server: NS3.GOOGLE.COM\n   Name Server: NS4.GOOGLE.COM\n   DNSSEC: unsigned\n   URL of the ICANN Whois Inaccuracy Complaint Form: https://www.icann.org/wicf/\n>>> Last update of whois database: 2021-03-27T21:37:18Z <<<\n\nFor more information on Whois status codes, please visit https://icann.org/epp\n\nNOTICE: The expiration date displayed in this record is the date the\nregistrar\'s sponsorship of the domain name registration in the registry is\ncurrently set to expire. This date does not necessarily reflect the expiration\ndate of the domain name registrant\'s agreement with the sponsoring\nregistrar.  Users may consult the sponsoring registrar\'s Whois database to\nview the registrar\'s reported date of expiration for this registration.\n\nTERMS OF USE: You are not authorized to access or query our Whois\ndatabase through the use of electronic processes that are high-volume and\nautomated except as reasonably necessary to register domain names or\nmodify existing registrations; the Data in VeriSign Global Registry\nServices\' ("VeriSign") Whois database is provided by VeriSign for\ninformation purposes only, and to assist persons in obtaining information\nabout or related to a domain name registration record. VeriSign does not\nguarantee its accuracy. By submitting a Whois query, you agree to abide\nby the following terms of use: You agree that you may use this Data only\nfor lawful purposes and that under no circumstances will you use this Data\nto: (1) allow, enable, or otherwise support the transmission of mass\nunsolicited, commercial advertising or solicitations via e-mail, telephone,\nor facsimile; or (2) enable high volume, automated, electronic processes\nthat apply to VeriSign (or its computer systems). The compilation,\nrepackaging, dissemination or other use of this Data is expressly\nprohibited without the prior written consent of VeriSign. You agree not to\nuse electronic processes that are automated and high-volume to access or\nquery the Whois database except as reasonably necessary to register\ndomain names or modify existing registrations. VeriSign reserves the right\nto restrict your access to the Whois database in its sole discretion to ensure\noperational stability.  VeriSign may restrict or terminate your access to the\nWhois database for failure to abide by these terms of use. VeriSign\nreserves the right to modify these terms at any time.\n\nThe Registry database contains ONLY .COM, .NET, .EDU domains and\nRegistrars.\n']}`
    """
    try:
        domain_whois = get_whois(domain)
        # print(domain_whois)
        return domain_whois
    except:
        return False


def get_domain_whois_expiration_date_legacy(domain):
    """Get a domains whois expiration date via legacy method via whois socket.

    :param domain: Domain to lookup whois expiration for
    :type domain: str
    :return: Dictionary of domain whois information
    :rtype: dict, bool
    :Example:
    >>> print(get_domain_whois_expiration_date_legacy('google.com'))
    2028-09-14 04:00:00
    """
    try:
        domain_whois = get_whois(domain)
        # print(domain_whois)
        # print(str(domain_whois["expiration_date"][0]))
        # return parse_date_convert(str(domain_whois["expiration_date"][0]))
        return str(domain_whois["expiration_date"][0])
    except:
        return False


def get_domain_rdap_info(domain):
    """Do WHOIS lookup via rdap method.

    :param domain: Domain to lookup whois for
    :type domain: str
    :return: Dictionary of domain whois information
    :rtype: dict, bool
    :Example:
    >>> print(get_domain_rdap_info('google.com'))
    `{'objectClassName': 'domain', 'handle': '2138514_DOMAIN_COM-VRSN', 'ldhName': 'GOOGLE.COM', 'links': [{'value': 'https://rdap.verisign.com/com/v1/domain/GOOGLE.COM', 'rel': 'self', 'href': 'https://rdap.verisign.com/com/v1/domain/GOOGLE.COM', 'type': 'application/rdap+json'}, {'value': 'https://rdap.markmonitor.com/rdap/domain/GOOGLE.COM', 'rel': 'related', 'href': 'https://rdap.markmonitor.com/rdap/domain/GOOGLE.COM', 'type': 'application/rdap+json'}], 'status': ['client delete prohibited', 'client transfer prohibited', 'client update prohibited', 'server delete prohibited', 'server transfer prohibited', 'server update prohibited'], 'entities': [{'objectClassName': 'entity', 'handle': '292', 'roles': ['registrar'], 'publicIds': [{'type': 'IANA Registrar ID', 'identifier': '292'}], 'vcardArray': ['vcard', [['version', {}, 'text', '4.0'], ['fn', {}, 'text', 'MarkMonitor Inc.']]], 'entities': [{'objectClassName': 'entity', 'roles': ['abuse'], 'vcardArray': ['vcard', [['version', {}, 'text', '4.0'], ['fn', {}, 'text', ''], ['tel', {'type': 'voice'}, 'uri', 'tel:+1.2083895740'], ['email', {}, 'text', 'abusecomplaints@markmonitor.com']]]}]}], 'events': [{'eventAction': 'registration', 'eventDate': '1997-09-15T04:00:00Z'}, {'eventAction': 'expiration', 'eventDate': '2028-09-14T04:00:00Z'}, {'eventAction': 'last update of RDAP database', 'eventDate': '2021-03-27T09:24:47Z'}], 'secureDNS': {'delegationSigned': False}, 'nameservers': [{'objectClassName': 'nameserver', 'ldhName': 'NS1.GOOGLE.COM'}, {'objectClassName': 'nameserver', 'ldhName': 'NS2.GOOGLE.COM'}, {'objectClassName': 'nameserver', 'ldhName': 'NS3.GOOGLE.COM'}, {'objectClassName': 'nameserver', 'ldhName': 'NS4.GOOGLE.COM'}], 'rdapConformance': ['rdap_level_0', 'icann_rdap_technical_implementation_guide_0', 'icann_rdap_response_profile_0'], 'notices': [{'title': 'Terms of Use', 'description': ['Service subject to Terms of Use.'], 'links': [{'href': 'https://www.verisign.com/domain-names/registration-data-access-protocol/terms-service/index.xhtml', 'type': 'text/html'}]}, {'title': 'Status Codes', 'description': ['For more information on domain status codes, please visit https://icann.org/epp'], 'links': [{'href': 'https://icann.org/epp', 'type': 'text/html'}]}, {'title': 'RDDS Inaccuracy Complaint Form', 'description': ['URL of the ICANN RDDS Inaccuracy Complaint Form: https://icann.org/wicf'], 'links': [{'href': 'https://icann.org/wicf', 'type': 'text/html'}]}]}`
    """
    request = rdapbootstrapurl + "domain/" + domain
    try:
        domain_response = http.get(request).text
        # print(request)
        # print(domain_response)
        domain_whois = json.loads(str(domain_response))
        # print(json.dumps(self.domain_whois, indent=4))
        return domain_whois
    except:
        print("RDAP Lookup Failed")
        return False


def get_domain_whois_event_date_rdap(domain, event_action, rdap_dict=None):
    """Get Domain whois event date from rdap.

    :param domain: Domain
    :type domain: str
    :param event_action: str Supported events: expiration, registration
    :type event_action: str
    :return: Date formatted "%Y-%m-%d %H:%M:%S" Example: '1997-09-15 04:00:00' or False
    :rtype: str
    :Example:
    >>> print(get_domain_whois_event_date_rdap('google.com', 'expiration'))
    2028-09-14 04:00:00
    """
    if rdap_dict is None:
        domain_whois = get_domain_rdap_info(domain)
    else:
        domain_whois = rdap_dict

    if domain_whois:
        for event in domain_whois["events"]:
            # print(event["eventAction"], event["eventDate"])
            if event["eventAction"] == event_action:
                # print(str(event["eventDate"]))
                return parse_date_convert(str(event["eventDate"]))
    else:
        return False


def get_domain_whois_expiration_date_rdap(domain):
    """Get domain whois expiration date from rdap.

    :param domain: Domain
    :type domain: str
    :return: Date formatted "%Y-%m-%d %H:%M:%S" Example: '1997-09-15 04:00:00' or False
    :rtype: str
    :Example:
    >>> print(get_domain_whois_expiration_date_rdap("google.com"))
    2028-09-14 04:00:00
    """
    return get_domain_whois_event_date_rdap(domain, "expiration")


def get_domain_whois_registration_date_rdap(domain):
    """Get domain whois registration date from rdap.

    :param domain: Domain
    :type domain: str
    :return: Date formatted "%Y-%m-%d %H:%M:%S" Example: '1997-09-15 04:00:00' or False
    :rtype: str
    :Example:
    >>> print(get_domain_whois_registration_date_rdap("google.com"))
    1997-09-15 04:00:00
    """
    return get_domain_whois_event_date_rdap(domain, "registration")


def get_ns_records(name):
    """Get NS(Nameserver) records for the name provided.

    :param name: Domain to lookup NS records for.
    :type name: str
    :return: Returns a list of the sorted NS records for the domain otherwise `None`.
    :rtype: list
    :Example:
    >>> print(get_ns_records("google.com"))
    `['ns1.google.com', 'ns2.google.com', 'ns3.google.com', 'ns4.google.com']`
    """
    ns = []
    try:
        coro = query(str(name).lower(), 'NS')
        res_ns = loop.run_until_complete(coro)
        for elem in res_ns:
            ns.append(str(elem.host))
        ns = sorted(ns)
    except:
        ns = None
    return ns


def get_soa_record(name):
    """Get SOA(Start of Authority) record for the name provided.

    :param name: Domain to get SOA record for.
    :type name: str
    :return: Returns a dictionary with the SOA information otherwise `None`.
    :rtype: dict
    :Example:
    >>> print(get_soa_record("google.com"))
    `{'SOA': {'nsname': 'ns1.google.com', 'hostmaster': 'dns-admin.google.com', 'serial': '365213266', 'refresh': '900', 'retry': '900', 'expires': '1800', 'minttl': '60', 'ttl': '13'}}`
    """
    try:
        coro = query(str(name).lower(), 'SOA')
        res_soa = loop.run_until_complete(coro)
        soa_dict = {"SOA": {"nsname": str(res_soa.nsname), "hostmaster": str(res_soa.hostmaster),
                            "serial": str(res_soa.serial),
                            "refresh": str(res_soa.refresh), "retry": str(res_soa.retry),
                            "expires": str(res_soa.expires),
                            "minttl": str(res_soa.minttl), "ttl": str(res_soa.ttl)}}
    except:
        soa_dict = None
    return soa_dict


def get_a_record(name):
    """Get A record(IPv4 address) for the name provided.

    :param name: Hostname to find A record/IP for.
    :type name: str
    :return: Returns an A record also known as an IPv4 address otherwise `None`
    :rtype: str
    :Example:
    >>> print(get_a_record("google.com"))
    172.217.15.206
    """
    try:
        coro = query(str(name).lower(), 'A')
        result = loop.run_until_complete(coro)
        ip = str(result[0].host)
    except:
        ip = None
    return ip


def get_aaaa_record(name):
    """Get AAAA record(IPv6 address) for the name provided.

    :param name: Hostname to find AAAA record/IP for.
    :type name: str
    :return: Returns an AAAA record also known as an IPv6 address otherwise `None`
    :rtype: str
    :Example:
    >>> print(get_aaaa_record("google.com"))
    2607:f8b0:4008:813::200e
    """
    try:
        coro = query(str(name).lower(), 'AAAA')
        result = loop.run_until_complete(coro)
        ipv6 = str(result[0].host)
    except:
        ipv6 = None
    return ipv6


def get_cname_record(name):
    """Get A record(IPv4 address) for the name provided.

    :param name: Hostname to find A record/IP for.
    :type name: str
    :return: Returns an A record also known as an IPv4 address otherwise `None`
    :rtype: str
    :Example:
    >>> print(get_a_record("google.com"))
    172.217.15.206
    """
    try:
        coro = query(str(name).lower(), 'CNAME')
        result = loop.run_until_complete(coro)
        ip = str(result[0].host)
    except:
        ip = None
    return ip


def get_txt_records(name):
    """Get TXT records for the domain provided.

    :param name: Domain to find TXT record for.
    :type name: str
    :return: Returns a list of TXT records
    :rtype: list
    :Example:

    >>> print(get_txt_records("google.com"))
    `[['TXT', 'google.com', 'facebook-domain-verification=22rm551cu4k0ab0bxsw536tlds4h95'], ['TXT', 'google.com', 'apple-domain-verification=30afIBcvSuDV2PLX'], ['TXT', 'google.com', 'v=spf1 include:_spf.google.com ~all'], ['TXT', 'google.com', 'globalsign-smime-dv=CDYX+XFHUw2wml6/Gb8+59BsH31KzUr6c1l2BPvqKX8='], ['TXT', 'google.com', 'docusign=1b0a6754-49b1-4db5-8540-d2c12664b289'], ['TXT', 'google.com', 'docusign=05958488-4752-4ef2-95eb-aa7ba8a3bd0e'], ['TXT', 'google.com', 'google-site-verification=wD8N7i1JTNTkezJ49swvWW48f8_9xveREV4oB-0Hf5o']]`
    """
    txt_records = []
    try:
        res_txt = loop.run_until_complete(resolver.query(name, 'TXT'))
        for elem in res_txt:
            # print(str(elem.text))
            txt_records.append(['TXT', str(name), str(elem.text)])
    except:
        txt_records = None
    return txt_records


def get_mx_records(name):
    """Get MX records for the domain provided.

    :param name: Domain to find MX records for.
    :type name: str
    :return: Returns a list of MX records
    :rtype: list
    :Example:
    >>> print(get_mx_records("wizardassistant.com"))
    `[<ares_query_mx_result> host=mail.wizardassistant.com, priority=10, ttl=244, <ares_query_mx_result> host=zimbra.wizardassistant.com, priority=0, ttl=244]`
    """
    res_mx = []
    try:
        res_mx = loop.run_until_complete(resolver.query(name, 'MX'))
    except:
        pass
    return res_mx


def get_dkim_records(name, select=None):
    """Get DKIM records for a domain.

    :param name: Domain name
    :type name: str
    :param select: Optional: Selector that can be also searched for.
    :type select: str
    :return: Returns a dictionary of dkim selectors as keys and their records as values.
    :rtype: dict
    :Example:
    >>> print(str(get_dkim_records('wizardassistant.com')))
    {'default._domainkey.wizardassistant.com': 'v=DKIM1; h=sha256; k=rsa; p=MIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEAynF/0vOKHvSzZyDNW9wIbdv7D8zhPZ4624JYt9YvyIV3EcNRMX0RSi7KyLCVvjVrOoWXHx5NPoXt5OM2hRXb+cTUYAj4tijuDUzkRjmXFEbHhdrQFholM5FF/nb4i5MnSyVUrOGcj0Wxjmt1y1KQn5U/HSrZP0bk3FnjCHcQ62prCgFGsum44crI2efIm8yPwWFaDzDE/xjn218BY8PuMiNu4LYznmdfXDDJeRa7U+uuLAz0diiAYgwNPGiN6RYPum2yyixdDawxB1R3qWpGnAfGj9Tpl045ALJCjUsh7l7C3PmcSXc8Ez8WPCgYqs8zOKHwwYi3ILTH12x1JxicpwIDAQAB'}

    """
    dkim = {}
    # Common selectors for rapid enumeration
    dkim_selectors = ['default', 'dkim', 'dkim1', 'google', 'k1', 'k2', 'mail', 'selector1', 'selector2', 'zoho']

    # If one is provided add it to the list
    if select is not None:
        dkim_selectors.append(str(select))

    try:
        # Here we are checking all the popular and common DKIM selector names in a loop plus any specific one provided
        for selector in dkim_selectors:
            # default._domainkey.domain.com
            # DKIM query the host's DNS
            dkim_name = f'{selector}._domainkey.{name}'
            res_dkim = loop.run_until_complete(resolver.query(dkim_name, 'TXT'))
            for elem in res_dkim:
                if 'v=DKIM' in str(elem.text):
                    dkim[str(dkim_name)] = str(elem.text)
    except:
        # we pass here so that the loop can keep trying till its tried them all
        pass

    return dkim


def get_dmarc_record(name):
    """Get DMARC record for domain.

    :param name: Domain name to get DMARC record for.
    :type name: str
    :return: Returns the string form of the DMARC record value if successful otherwise `None`.
    :rtype: str
    :Example:
    >>> print(get_dmarc_record("google.com"))
    v=DMARC1; p=reject; rua=mailto:mailauth-reports@google.com
    """
    # _dmarc.domain.com
    dmarc_name = '_dmarc.' + name
    dmarc = None
    try:
        # DMARC query the host's DNS
        res_dmarc = loop.run_until_complete(resolver.query(dmarc_name, 'TXT'))

        for elem in res_dmarc:
            if 'v=DMARC' in str(elem.text):
                dmarc = str(elem.text)
    except:
        pass
    return dmarc


def check_domain_expired(expiration_date, fmt=None):
    """Check if domain expiration date has passed.
    :param expiration_date: Expiration datetime to check
    :type expiration_date: str
    :param fmt: Datetime format to convert to when checking
    :type fmt: datetime format string like: '%Y-%m-%d %H:%M:%S'
    :return: Returns True if expired
    :rtype: bool
    :Example:
    >>> print(check_domain_expired('2028-09-14 04:00:00'))
    False
    """
    if fmt is None:
        fmt = '%Y-%m-%d %H:%M:%S'
    try:
        past = datetime.strptime(str(expiration_date), fmt)
        present = datetime.now()
        if past.date() < present.date():
            # print('Domain is expired')
            return True
        else:
            # print('Domain is not expired')
            return False
    except:
        return True
        pass


# print(get_mx_records("wizardassistant.com"))
# print(check_domain_expired('2028-09-14 04:00:00'))
# print(get_dmarc_record("google.com"))
# print(str(get_dkim_records('wizardassistant.com')))

# print(parse_date_convert('2028-09-14T04:00:00Z'))
# print(get_domain_whois_info_legacy('wizardassistant.com'))
# print(get_domain_whois_expiration_date_legacy("wizardassistant.com"))
# print(get_domain_whois_info_legacy('wizardassistant.app'))
# print(get_domain_whois_expiration_date_legacy("google.com"))
# print(get_domain_whois_expiration_date_rdap("google.com"))
# print(get_a_record("google.com"))
# print(get_aaaa_record("google.com"))
# print(get_ns_records("google.com"))
# print(get_soa_record("google.com"))

# print(get_domain_whois_info_legacy("google.com.au"))
# print(get_domain_whois_expiration_date_rdap("google.com"))
# test commands
# print(get_hostname_from_ip("1.1.1.1"))
# print(get_rdns_from_ip("8.8.8.8"))
# print(get_domain_whois_info_legacy("google.com"))

# print(get_domain_rdap_info("google.com"))
# print(get_domain_whois_registration_date_rdap("google.com"))
# print(get_domain_whois_expiration_date_rdap("wizardassistant.com"))
# print(get_domain_whois_expiration_date_rdap("wizardassistant.app"))
# print(get_domain_whois_event_date_rdap("google.com", "expiration"))
# print(get_domain_whois_event_date_rdap("google.com", "registration"))
# print(get_domain_whois_expiration_date_rdap("google.com"))



"""
Wizard Domaininfo package.

DNS/Whois Domain Information library
"""

from typing import List
from wizard_domaininfo import net
from wizard_domaininfo import parse

__all__: List[str] = []  # noqa: WPS410 (the only __variable__ we use)


def get_whois(domain, normalized=[]):
    raw_data, server_list = net.get_whois_raw(domain, with_server_list=True)
    # Unlisted handles will be looked up on the last WHOIS server that was queried. This may be changed to also query
    # other servers in the future, if it turns out that there are cases where the last WHOIS server in the chain doesn't
    # actually hold the handle contact details, but another WHOIS server in the chain does.
    return parse.parse_raw_whois(raw_data, normalized=normalized, never_query_handles=False,
                                 handle_server=server_list[-1])


def whois(*args, **kwargs):
    raise Exception("The whois() method has been replaced by a different method (with a different API), "
                    "since wizard_whois 2.0. Either install the older wizard_whois 1.2.3, or change your code to use "
                    "the new API.")



import re
import socket
import sys
from codecs import encode, decode

from wizard_domaininfo import shared


def get_whois_raw(domain, server="", previous=None, rfc3490=True, never_cut=False, with_server_list=False, server_list=None):
    previous = previous or []
    server_list = server_list or []
    # Sometimes IANA simply won't give us the right root WHOIS server
    exceptions = {
        ".ac.uk": "whois.ja.net",
        ".gov.uk": 'whois.ja.net',
        ".ps": "whois.pnina.ps",
        ".buzz": "whois.nic.buzz",
        ".moe": "whois.nic.moe",
        # The following is a bit hacky, but IANA won't return the right answer for example.com because it's a direct
        # registration.
        "example.com": "whois.verisign-grs.com"
    }

    if rfc3490:
        if sys.version_info < (3, 0):
            domain = encode(domain if type(domain) is unicode else decode(domain, "utf8"), "idna")
        else:
            domain = encode(domain, "idna").decode("ascii")

    if len(previous) == 0 and server == "":
        # Root query
        is_exception = False
        for exception, exc_serv in exceptions.items():
            if domain.endswith(exception):
                is_exception = True
                target_server = exc_serv
                break
        if is_exception == False:
            target_server = get_root_server(domain)
    else:
        target_server = server
    if target_server == "whois.jprs.jp":
        request_domain = "%s/e" % domain  # Suppress Japanese output
    elif domain.endswith(".de") and (target_server == "whois.denic.de" or target_server == "de.whois-servers.net"):
        request_domain = "-T dn,ace %s" % domain  # regional specific stuff
    elif target_server == "whois.verisign-grs.com":
        request_domain = "=%s" % domain  # Avoid partial matches
    else:
        request_domain = domain
    response = whois_request(request_domain, target_server)
    if never_cut:
        # If the caller has requested to 'never cut' responses, he will get the original response from the server (
        # this is useful for callers that are only interested in the raw data). Otherwise, if the target is
        # verisign-grs, we will select the data relevant to the requested domain, and discard the rest, so that in a
        # multiple-option response the parsing code will only touch the information relevant to the requested domain.
        # The side-effect of this is that when `never_cut` is set to False, any verisign-grs responses in the raw
        # data will be missing header, footer, and alternative domain options (this is handled a few lines below,
        # after the verisign-grs processing).
        new_list = [response] + previous
    if target_server == "whois.verisign-grs.com":
        # VeriSign is a little... special. As it may return multiple full records and there's no way to do an exact
        # query, we need to actually find the correct record in the list.
        for record in response.split("\n\n"):
            if re.search("Domain Name: %s\n" % domain.upper(), record):
                response = record
                break
    if not never_cut:
        new_list = [response] + previous
    server_list.append(target_server)
    for line in [x.strip() for x in response.splitlines()]:
        match = re.match("(refer|whois server|referral url|whois server|registrar whois):\s*([^\s]+\.[^\s]+)", line, re.IGNORECASE)
        if match is not None:
            referal_server = match.group(2)
            if referal_server != server and "://" not in referal_server:  # We want to ignore anything non-WHOIS (eg.
                # HTTP) for now.
                # Referal to another WHOIS server...
                return get_whois_raw(domain, referal_server, new_list, server_list=server_list, with_server_list=with_server_list)
    if with_server_list:
        return new_list, server_list
    else:
        return new_list


def get_root_server(domain):
    data = whois_request(domain, "whois.iana.org")
    for line in [x.strip() for x in data.splitlines()]:
        match = re.match("refer:\s*([^\s]+)", line)
        if match is None:
            continue
        return match.group(1)
    raise shared.WhoisException("No root WHOIS server found for domain.")


def whois_request(domain, server, port=43, timeout=10):
    sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    sock.settimeout(timeout)
    sock.connect((server, port))
    sock.send(("%s\r\n" % domain).encode("utf-8"))
    buff = b""
    while True:
        data = sock.recv(1024)
        if len(data) == 0:
            break
        buff += data
    return buff.decode("utf-8")




# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['wizard_domaininfo']

package_data = \
{'': ['*']}

install_requires = \
['aiodns>=2.0.0,<3.0.0',
 'atomicwrites>=1.4.0,<2.0.0',
 'cached-property>=1.5.2,<2.0.0',
 'contextvars>=2.4,<3.0',
 'immutables>=0.15,<0.16',
 'importlib-metadata>=3.7.0,<4.0.0',
 'python-dateutil>=2.8.1,<3.0.0',
 'requests>=2.25.1,<3.0.0',
 'setuptools>=54.1.0,<55.0.0',
 'tomlkit>=0.7.0,<0.8.0',
 'typing>=3.7.4,<4.0.0',
 'wheel>=0.36.2,<0.37.0',
 'zipp>=3.4.1,<4.0.0']

entry_points = \
{'console_scripts': ['pwhois = wizard_domaininfo:pwhois',
                     'wizard-domaininfo = wizard_domaininfo.cli:main']}

setup_kwargs = {
    'name': 'wizard-domaininfo',
    'version': '0.2.2',
    'description': 'DNS/Whois Domain Information library',
    'long_description': '# Wizard Domaininfo\n\n[![pipeline status](https://gitlab.com/mikeramsey/wizard-domaininfo/badges/master/pipeline.svg)](https://gitlab.com/mikeramsey/wizard-domaininfo/pipelines)\n[![coverage report](https://gitlab.com/mikeramsey/wizard-domaininfo/badges/master/coverage.svg)](https://gitlab.com/mikeramsey/wizard-domaininfo/commits/master)\n[![documentation](https://img.shields.io/badge/docs-mkdocs%20material-blue.svg?style=flat)](https://mikeramsey.gitlab.io/wizard-domaininfo/)\n[![pypi version](https://img.shields.io/pypi/v/wizard-domaininfo.svg)](https://pypi.org/project/wizard-domaininfo/)\n[![gitter](https://badges.gitter.im/join%20chat.svg)](https://gitter.im/wizard-domaininfo/community)\n\nDNS/Whois Domain Information library\n\n## Requirements\n\nWizard Domaininfo requires Python 3.6 or above.\n\n<details>\n<summary>To install Python 3.6, I recommend using <a href="https://github.com/pyenv/pyenv"><code>pyenv</code></a>.</summary>\n\n```bash\n# install pyenv\ngit clone https://github.com/pyenv/pyenv ~/.pyenv\n\n# setup pyenv (you should also put these three lines in .bashrc or similar)\nexport PATH="${HOME}/.pyenv/bin:${PATH}"\nexport PYENV_ROOT="${HOME}/.pyenv"\neval "$(pyenv init -)"\n\n# install Python 3.6\npyenv install 3.6.12\n\n# make it available globally\npyenv global system 3.6.12\n```\n</details>\n\n## Installation\n\nWith `pip`:\n```bash\npython3.6 -m pip install wizard_domaininfo\n```\n\nWith [`pipx`](https://github.com/pipxproject/pipx):\n```bash\npython3.6 -m pip install --user pipx\n\npipx install --python python3.6 wizard_domaininfo\n```\n\n=========\n\nA WHOIS DNS retrieval and parsing library for Python.\n\n## Dependencies\n\nNone! All you need is the Python standard library. Optional RDAP WHOIS based lookups require requests and json libraries. Optional DNS lookups are based on aiodns library.\n\n## Instructions\n\nThe whois legacy manual HTML version is also viewable [here](http://cryto.net/pythonwhois).\nThe new manual for RDAP based whois and DNS is located [here](https://mikeramsey.gitlab.io/wizard-domaininfo/).\n## Goals\n\n* 100% coverage of WHOIS/DNS formats.\n* Accurate and complete data.\n* Consistently functional parsing; constant tests to ensure the parser isn\'t accidentally broken.\n\n## Features\n\n* WHOIS data retrieval\n\t* Able to follow WHOIS server redirects\n\t* Won\'t get stuck on multiple-result responses from verisign-grs\n* WHOIS data parsing\n\t* Base information (registrar, etc.)\n\t* Dates/times (registration, expiry, ...)\n\t* Full registrant information (!)\n\t* Nameservers\n* Optional WHOIS data normalization\n\t* Attempts to intelligently reformat WHOIS data for better (human) readability\n\t* Converts various abbreviation types to full locality names\n\t\t* Airport codes\n\t\t* Country names (2- and 3-letter ISO codes)\n\t\t* US states and territories\n\t\t* Canadian states and territories\n\t\t* Australian states\n* `pwhois`, a simple WHOIS tool using pythonwhois\n\t* Easily readable output format\n\t* Can also output raw WHOIS data\n\t* ... and JSON.\n* Automated testing suite for parse.py and legacy whois(non rdap based)\n\t* Will detect and warn about any changes in parsed data compared to previous runs\n\t* Guarantees that previously working WHOIS parsing doesn\'t unintentionally break when changing code\n* Automated testing suite for rdap and aiodns is based on pytest\n* Fast Asynchronous DNS lookups via c-ares aiodns library and helper methods in utils.py.\n* Single call via DomainInfo class in domaininfo.py will get all available DNS/Whois via rdap and fallback to legacy whois as needed.\n\t* Checks for WAFs(Cloudflare/Sucuri/Quic.Cloud)\n\t* Check domain expiration.\n\t* Check for or enumerate DKIM records and selectors.\n\t* Check Whois Nameservers and DNS nameservers match.\n\t* See [here](https://gitlab.com/mikeramsey/wizard-domaininfo/-/blob/master/src/wizard_domaininfo/domaininfo.py#L23-69) for all the class attributes.\n\t\n\n## Important update notes\n\n\n## It doesn\'t work!\n\n* It doesn\'t work at all?\n* It doesn\'t parse the data for a particular domain?\n* There\'s an inaccuracy in parsing the data for a domain, even just a small one?\n\nIf any of those apply, don\'t hesitate to file an issue! The goal is 100% coverage, and we need your feedback to reach that goal.\n\n## License\n\nThis library may be used under the MIT License.\n\n## Data sources\n\nThis library uses a number of third-party datasets for normalization:\n\n* `airports.dat`: [OpenFlights Airports Database](http://openflights.org/data.html) ([Open Database License 1.0](http://opendatacommons.org/licenses/odbl/1.0/), [Database Contents License 1.0](http://opendatacommons.org/licenses/dbcl/1.0/))\n* `countries.dat`: [Country List](https://github.com/umpirsky/country-list) (MIT license)\n* `countries3.dat`: [ISO countries list](https://gist.github.com/eparreno/205900) (license unspecified)\n* `states_au.dat`: Part of `pythonwhois` (WTFPL/CC0)\n* `states_us.dat`: [State Table](http://statetable.com/) (license unspecified, free reuse encouraged)\n* `states_ca.dat`: [State Table](http://statetable.com/) (license unspecified, free reuse encouraged)\n\nBe aware that the OpenFlights database in particular has potential licensing consequences; if you do not wish to be bound by these potential consequences, you may simply delete the `airports.dat` file from your distribution. `pythonwhois` will assume there is no database available, and will not perform airport code conversion (but still function correctly otherwise). This also applies to other included datasets.\n\n## Contributing\n\nFeel free to fork and submit pull requests (to the `develop` branch)! If you change any parsing or normalization logic, ensure to run the full test suite before opening a pull request. Instructions for that are below.\n\nPlease note that this project uses tabs for indentation.\n\nAll commands are relative to the root directory of the repository.\n\n**Pull requests that do _not_ include output from test_parse.py will be rejected!**\n\n### Adding new WHOIS data to the testing set\n\n\tpwhois --raw thedomain.com > test/data/thedomain.com\n\t\n### Checking the currently parsed data (while editing the parser)\n\n\t./pwhois -f test/data/thedomain.com/ .\n\t\n(don\'t forget the dot at the end!)\n\t\n### Marking the current parsed data as correct for a domain\n\nMake sure to verify (using `pwhois` or otherwise) that the WHOIS data for the domain is being parsed correctly, before marking it as correct!\n\n\tpython test_parse.py update thedomain.com\n\t\n### Running all tests\n\n\tpython test_parse.py run all\n\t\n### Testing a specific domain\n\n\tpython test_parse.py run thedomain.com',
    'author': 'Michael Ramsey',
    'author_email': 'mike@hackerdise.me',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://gitlab.com/mikeramsey/wizard-domaininfo',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.6.1,<4.0.0',
}


setup(**setup_kwargs)
